"""Modern Forms Constants."""
DEFAULT_TIMEOUT_SECS = 5
DEFAULT_PORT = 80
DEFAULT_API_ENDPOINT = "mf"

COMMAND_QUERY_STATUS = "queryDynamicShadowData"
COMMAND_QUERY_STATIC_DATA = "queryStaticShadowData"
COMMAND_FAN_POWER = "fanOn"
COMMAND_LIGHT_POWER = "lightOn"
COMMAND_LIGHT_BRIGHTNESS = "lightBrightness"
COMMAND_FAN_SPEED = "fanSpeed"
COMMAND_FAN_DIRECTION = "fanDirection"
COMMAND_AWAY_MODE = "awayModeEnabled"
COMMAND_FAN_SLEEP_TIMER = "fanSleepTimer"
COMMAND_LIGHT_SLEEP_TIMER = "lightSleepTimer"
COMMAND_ADAPTIVE_LEARNING = "adaptiveLearning"
COMMAND_REBOOT = "reboot"

AWAY_MODE_OFF = False
AWAY_MODE_ON = True

ADAPTIVE_LEARNING_OFF = False
ADAPTIVE_LEARNING_ON = True

FAN_DIRECTION_FORWARD = "forward"
FAN_DIRECTION_REVERSE = "reverse"

FAN_POWER_OFF = False
FAN_POWER_ON = True

LIGHT_POWER_OFF = False
LIGHT_POWER_ON = True

LIGHT_BRIGHTNESS_LOW_VALUE = 1
LIGHT_BRIGHTNESS_HIGH_VALUE = 100

FAN_SPEED_LOW_VALUE = 1
FAN_SPEED_HIGH_VALUE = 6

STATE_FAN_POWER = "fanOn"
STATE_LIGHT_POWER = "lightOn"
STATE_LIGHT_BRIGHTNESS = "lightBrightness"
STATE_FAN_SPEED = "fanSpeed"
STATE_FAN_DIRECTION = "fanDirection"
STATE_AWAY_MODE = "awayModeEnabled"
STATE_FAN_SLEEP_TIMER = "fanSleepTimer"
STATE_LIGHT_SLEEP_TIMER = "lightSleepTimer"
STATE_ADAPTIVE_LEARNING = "adaptiveLearning"


INFO_CLIENT_ID = "clientId"
INFO_MAC = "mac"
INFO_LIGHT_TYPE = "lightType"
INFO_FAN_TYPE = "fanType"
INFO_FAN_MOTOR_TYPE = "fanMotorType"
INFO_PRODUCTION_LOT_NUMBER = "productionLotNumber"
INFO_PRODUCT_SKU = "productSku"
INFO_OWNER = "owner"
INFO_FEDERATED_IDENTITY = "federatedIdentity"
INFO_DEVICE_NAME = "deviceName"
INFO_FIRMWARE_VERSION = "firmwareVersion"
INFO_MAIN_MCU_FIRMWARE_VERSION = "mainMcuFirmwareVersion"
INFO_FIRMWARE_URL = "firmwareUrl"

SLEEP_TIMER_CANCEL = 0
